<?php
/**
 * Plugin Name: DOM Optimizer (EasyToolsWP)
 * Description: Sort the elements of the <head> into the correct order using DOM manipulation. Allows you to move styles & scripts to the footer.
 * Version: 1.2.1
 * Author: EasyToolsWP.com
 */

function etwp_dom_add_settings_menu() {
    add_options_page(
        'DOM Optimizer',      
        'DOM Optimizer',      
        'manage_options',     
        'etwp-dom-optimizer', 
        'etwp_dom_settings_page' 
    );
}
add_action('admin_menu', 'etwp_dom_add_settings_menu');

function etwp_dom_settings_init() {
    register_setting('etwp_dom_settings', 'etwp_dom_enable');
    register_setting('etwp_dom_settings', 'etwp_dom_move_styles');
    register_setting('etwp_dom_settings', 'etwp_dom_move_scripts');
    register_setting('etwp_dom_settings', 'etwp_dom_move_links');

    add_settings_section(
        'etwp_dom_section', 
        __('DOM Optimizer Settings', 'textdomain'), 
        null, 
        'etwp-dom-optimizer'
    );

    add_settings_field(
        'etwp_dom_enable', 
        __('Enable DOM sorting', 'textdomain'), 
        'etwp_dom_enable_render', 
        'etwp-dom-optimizer', 
        'etwp_dom_section'
    );

    add_settings_field(
        'etwp_dom_move_styles', 
        __('Move all styles to footer', 'textdomain'), 
        'etwp_dom_move_styles_render', 
        'etwp-dom-optimizer', 
        'etwp_dom_section'
    );

    add_settings_field(
        'etwp_dom_move_scripts', 
        __('Move all scripts to footer', 'textdomain'), 
        'etwp_dom_move_scripts_render', 
        'etwp-dom-optimizer', 
        'etwp_dom_section'
    );

    add_settings_field(
        'etwp_dom_move_links',  
        __('Move all links to footer', 'textdomain'), 
        'etwp_dom_move_links_render', 
        'etwp-dom-optimizer', 
        'etwp_dom_section'
    );
}
add_action('admin_init', 'etwp_dom_settings_init');

function etwp_dom_enable_render() {
    $option = get_option('etwp_dom_enable');
    ?>
    <input type="checkbox" name="etwp_dom_enable" value="1" <?php checked(1, $option, true); ?>>
    <?php
}

function etwp_dom_move_styles_render() {
    $option = get_option('etwp_dom_move_styles');
    ?>
    <input type="checkbox" name="etwp_dom_move_styles" value="1" <?php checked(1, $option, true); ?>>
    <?php
}

function etwp_dom_move_scripts_render() {
    $option = get_option('etwp_dom_move_scripts');
    ?>
    <input type="checkbox" name="etwp_dom_move_scripts" value="1" <?php checked(1, $option, true); ?>>
    <?php
}

function etwp_dom_move_links_render() {
    $option = get_option('etwp_dom_move_links');
    ?>
    <input type="checkbox" name="etwp_dom_move_links" value="1" <?php checked(1, $option, true); ?>>
    <?php
}

function etwp_dom_settings_page() {
    ?>
    <form action="options.php" method="post">
        <?php
        settings_fields('etwp_dom_settings');
        do_settings_sections('etwp-dom-optimizer');
        submit_button();
        ?>
    </form>
    <?php
}

function etwp_dom_sort_head_content($buffer) {
    if (!get_option('etwp_dom_enable')) {
        return $buffer;
    }

    $dom = new DOMDocument();
    @$dom->loadHTML($buffer);

    $head = $dom->getElementsByTagName('head')->item(0);
    $body = $dom->getElementsByTagName('body')->item(0);

    if ($head && $body) {
        $elements = [
            'meta' => [],
            'title' => [],
            'link' => [],
            'style' => [],
            'script' => []
        ];

        foreach ($head->childNodes as $node) {
            if ($node->nodeType === XML_ELEMENT_NODE && isset($elements[$node->nodeName])) {
                $elements[$node->nodeName][] = $node;
            }
        }

        while ($head->firstChild) {
            $head->removeChild($head->firstChild);
        }

        if (get_option('etwp_dom_move_styles')) {
            foreach ($elements['style'] as $style) {
                $body->appendChild($style);
            }
            unset($elements['style']);
        }

        if (get_option('etwp_dom_move_scripts')) {
            foreach ($elements['script'] as $script) {
                $body->appendChild($script);
            }
            unset($elements['script']);
        }

        if (get_option('etwp_dom_move_links')) {
            foreach ($elements['link'] as $link) {
                $body->appendChild($link);
            }
            unset($elements['link']);
        }

        foreach (['meta', 'title', 'link', 'style', 'script'] as $tag) {
            if (isset($elements[$tag])) {
                foreach ($elements[$tag] as $element) {
                    $head->appendChild($element);
                }
            }
        }

        $buffer = $dom->saveHTML();
    }

    return $buffer;
}

function etwp_dom_start_buffer() {
    ob_start('etwp_dom_sort_head_content');
}

function etwp_dom_end_buffer() {
    ob_end_flush();
}

add_action('wp_head', 'etwp_dom_start_buffer', 1);
add_action('wp_footer', 'etwp_dom_end_buffer', 100);